@@ -45,6 +45,13 @@
 static ETreeModel *parent_class;
 static GMemChunk  *node_chunk;
 
+enum {
+	NODE_RESORTED,
+	LAST_SIGNAL
+};
+
+static guint signals [LAST_SIGNAL] = {0, };
+
 typedef struct ETreeSortedPath ETreeSortedPath;
 
 struct ETreeSortedPath {
@@ -114,6 +121,17 @@ ets_sort_idle(gpointer user_data)
 	return FALSE;
 }
 
+#define ETS_SORT_IDLE_ACTIVATED(ets) ((ets)->priv->sort_idle_id != 0)
+
+inline static void
+ets_stop_sort_idle (ETreeSorted *ets)
+{
+	if (ets->priv->sort_idle_id) {
+		g_source_remove(ets->priv->sort_idle_id);
+		ets->priv->sort_idle_id = 0;
+	}
+}
+
 static gboolean
 ets_insert_idle(ETreeSorted *ets)
 {
@@ -381,6 +399,7 @@ reposition_path (ETreeSorted *ets, ETreeSortedPath *path)
 					for (i = old_index; i <= new_index; i++)
 						parent->children[i]->position = i;
 					e_tree_model_node_changed(E_TREE_MODEL(ets), parent);
+					e_tree_sorted_node_resorted(ets, parent);
 					e_tree_model_pre_change(E_TREE_MODEL(ets));
 				} else if (new_index < old_index) {
 					int i;
@@ -390,6 +409,7 @@ reposition_path (ETreeSorted *ets, ETreeSortedPath *path)
 					for (i = new_index; i <= old_index; i++)
 						parent->children[i]->position = i;
 					e_tree_model_node_changed(E_TREE_MODEL(ets), parent);
+					e_tree_sorted_node_resorted(ets, parent);
 					e_tree_model_pre_change(E_TREE_MODEL(ets));
 				}
 			}
@@ -481,8 +501,10 @@ resort_node (ETreeSorted *ets, ETreeSortedPath *path, gboolean resort_all_childr
 		path->child_needs_resort = 0;
 		path->needs_regen_to_sort = 0;
 		path->resort_all_children = 0;
-		if (needs_resort && send_signals && path->num_children > 0)
+		if (needs_resort && send_signals && path->num_children > 0) {
 			e_tree_model_node_changed(E_TREE_MODEL(ets), path);
+			e_tree_sorted_node_resorted(ets, path);
+		}
 	}
 }
 
@@ -542,7 +564,8 @@ ets_destroy (GtkObject *object)
 
 	/* FIXME lots of stuff to free here */
 
-	free_path(priv->root);
+	if (priv->root)
+		free_path(priv->root);
 
 	if (priv->source) {
 		gtk_signal_disconnect (GTK_OBJECT (priv->source),
@@ -579,10 +602,7 @@ ets_destroy (GtkObject *object)
 		priv->sort_info_changed_id = 0;
 	}
 
-	if (ets->priv->sort_idle_id) {
-		g_source_remove(ets->priv->sort_idle_id);
-		ets->priv->sort_idle_id = 0;
-	}
+	ets_stop_sort_idle (ets);
 	if (ets->priv->insert_idle_id) {
 		g_source_remove(ets->priv->insert_idle_id);
 		ets->priv->insert_idle_id = 0;
@@ -846,7 +866,7 @@ ets_has_get_node_by_id (ETreeModel *etm)
 }
 
 static ETreePath
-ets_get_node_by_id (ETreeModel *etm, gchar *save_id)
+ets_get_node_by_id (ETreeModel *etm, const char *save_id)
 {
 	ETreeSorted *ets = E_TREE_SORTED(etm);
 	ETreePath node;
@@ -951,6 +971,8 @@ ets_proxy_node_changed (ETreeModel *etm, ETreePath node, ETreeSorted *ets)
 	d(g_print("Setting last access %p. (ets_proxy_node_changed)\n", ets->priv->last_access));
 
 	if (e_tree_model_node_is_root(ets->priv->source, node)) {
+		ets_stop_sort_idle (ets);
+
 		if (ets->priv->root) {
 			free_path(ets->priv->root);
 		}
@@ -1017,7 +1039,7 @@ ets_proxy_node_inserted (ETreeModel *etm, ETreePath parent, ETreePath child, ETr
 		i = parent_path->num_children;
 		path = new_path(parent_path, child);
 		path->orig_position = position;
-		if (ets->priv->sort_idle_id == 0) {
+		if (!ETS_SORT_IDLE_ACTIVATED (ets)) {
 			ets->priv->insert_count++;
 			if (ets->priv->insert_count > ETS_INSERT_MAX) {
 				/* schedule a sort, and append instead */
@@ -1102,17 +1124,20 @@ ets_sort_info_changed (ETableSortInfo *sort_info, ETreeSorted *ets)
 /* Initialization and creation */
 
 static void
-e_tree_sorted_class_init (GtkObjectClass *klass)
+e_tree_sorted_class_init (ETreeSortedClass *klass)
 {
-	ETreeModelClass *tree_class      = (ETreeModelClass *) klass;
+	ETreeModelClass *tree_class      = E_TREE_MODEL_CLASS (klass);
+	GtkObjectClass *object_class     = GTK_OBJECT_CLASS (klass);
 
 	parent_class                     = gtk_type_class (PARENT_TYPE);
 
 	node_chunk                       = g_mem_chunk_create (ETreeSortedPath, TREEPATH_CHUNK_AREA_SIZE, G_ALLOC_AND_FREE);
+
+	klass->node_resorted             = NULL;
 	
-	klass->destroy                   = ets_destroy;
-	klass->set_arg                   = ets_set_arg;
-	klass->get_arg                   = ets_get_arg;
+	object_class->destroy            = ets_destroy;
+	object_class->set_arg            = ets_set_arg;
+	object_class->get_arg            = ets_get_arg;
 
 	tree_class->get_root             = ets_get_root;
 	tree_class->get_parent           = ets_get_parent;
@@ -1154,6 +1179,16 @@ e_tree_sorted_class_init (GtkObjectClass *klass)
 
 	gtk_object_add_arg_type ("ETreeSorted::sort_info", E_TABLE_SORT_INFO_TYPE,
 				 GTK_ARG_READWRITE, ARG_SORT_INFO);
+
+	signals [NODE_RESORTED] =
+		gtk_signal_new ("node_resorted",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (object_class),
+				GTK_SIGNAL_OFFSET (ETreeSortedClass, node_resorted),
+				gtk_marshal_NONE__POINTER,
+				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, signals, LAST_SIGNAL);
 }
 
 static void
@@ -1278,3 +1313,13 @@ e_tree_sorted_node_num_children   (ETreeSorted    *ets,
 	ETreeSortedPath *sorted_path = path;
 	return sorted_path->num_children;
 }
+
+void
+e_tree_sorted_node_resorted  (ETreeSorted *sorted, ETreePath node)
+{
+	g_return_if_fail (sorted != NULL);
+	g_return_if_fail (E_IS_TREE_SORTED (sorted));
+	
+	gtk_signal_emit (GTK_OBJECT (sorted),
+			 signals [NODE_RESORTED], node);
+}
