@@ -553,14 +553,17 @@ etta_class_init (ETreeTableAdapterClass *klass)
 	object_class->destroy           = etta_destroy;
 
 	table_class->column_count       = etta_column_count;
-	table_class->has_save_id        = etta_has_save_id;
-	table_class->get_save_id        = etta_get_save_id;
-	table_class->has_change_pending = etta_has_change_pending;
 	table_class->row_count          = etta_row_count;
+	table_class->append_row         = etta_append_row;
+
 	table_class->value_at           = etta_value_at;
 	table_class->set_value_at       = etta_set_value_at;
 	table_class->is_cell_editable   = etta_is_cell_editable;
-	table_class->append_row         = etta_append_row;
+
+	table_class->has_save_id        = etta_has_save_id;
+	table_class->get_save_id        = etta_get_save_id;
+
+	table_class->has_change_pending = etta_has_change_pending;
 	table_class->duplicate_value    = etta_duplicate_value;
 	table_class->free_value         = etta_free_value;
 	table_class->initialize_value   = etta_initialize_value;
@@ -605,21 +608,18 @@ etta_proxy_node_changed (ETreeModel *etm, ETreePath path, ETreeTableAdapter *ett
 		int row = find_row_num(etta, path);
 		int size;
 		int old_size;
+		ETreeTableAdapterNode *node;
 
 		if (row == -1)
 			return;
 
 		size = array_size_from_path(etta, path);
-		if (e_tree_model_node_is_expandable(etta->priv->source, path)) {
-			ETreeTableAdapterNode *node = find_or_create_node(etta, path);
+
+		node = find_node(etta, path);
+		if (node)
 			old_size = node->num_visible_children + 1;
-		} else {
-			ETreeTableAdapterNode *node = find_node(etta, path);
-			if (node)
-				old_size = node->num_visible_children + 1;
-			else
-				old_size = 1;
-		}
+		else
+			old_size = 1;
 
 		etta_expand_to(etta, etta->priv->n_map + size - old_size);
 
@@ -886,7 +886,7 @@ e_tree_table_adapter_load_expanded_state (ETreeTableAdapter *etta, const char *f
 		return;
 	}
 
-	for (child = root->childs; child; child = child->next) {
+	for (child = root->xmlChildrenNode; child; child = child->next) {
 		char *id;
 
 		if (strcmp (child->name, "node")) {
@@ -928,14 +928,17 @@ void         e_tree_table_adapter_node_set_expanded (ETreeTableAdapter *etta, ET
 	ETreeTableAdapterNode *node;
 	int row;
 
+	if (e_tree_model_node_is_root (etta->priv->source, path) && !etta->priv->root_visible)
+		return;
+
 	node = find_or_create_node(etta, path);
-	row = find_row_num(etta, path);
 
 	if (expanded != node->expanded) {
 		e_table_model_pre_change (E_TABLE_MODEL(etta));
 
 		node->expanded = expanded;
 		
+		row = find_row_num(etta, path);
 		if (row != -1) {
 			if (etta->priv->root_visible)
 				e_table_model_row_changed(E_TABLE_MODEL(etta), row);
@@ -958,6 +961,7 @@ void         e_tree_table_adapter_node_set_expanded (ETreeTableAdapter *etta, ET
 				}
 			} else {
 				int num_children = node->num_visible_children;
+				g_assert (etta->priv->n_map >= row + 1 + num_children);
 				memmove(etta->priv->map_table + row + 1,
 					etta->priv->map_table + row + 1 + num_children,
 					(etta->priv->n_map - row - 1 - num_children) * sizeof (ETreePath));
@@ -989,18 +993,29 @@ void         e_tree_table_adapter_node_set_expanded_recurse (ETreeTableAdapter *
 
 ETreePath    e_tree_table_adapter_node_at_row (ETreeTableAdapter *etta, int row)
 {
-	if (etta->priv->root_visible)
-		return etta->priv->map_table[row];
-	else
-		return etta->priv->map_table[row + 1];
+	if (row < 0)
+		return NULL;
+	if (etta->priv->root_visible) {
+		if (row < etta->priv->n_map)
+			return etta->priv->map_table[row];
+	} else {
+		if (row + 1 < etta->priv->n_map)
+			return etta->priv->map_table[row + 1];
+	}
+	return NULL;
 }
 
 int    e_tree_table_adapter_row_of_node (ETreeTableAdapter *etta, ETreePath path)
 {
 	if (etta->priv->root_visible)
 		return find_row_num(etta, path);
-	else
-		return find_row_num(etta, path) - 1;
+	else {
+		int row_num = find_row_num (etta, path);
+		if (row_num != -1)
+			return row_num - 1;
+		else
+			return row_num;
+	}
 }
 
 gboolean     e_tree_table_adapter_root_node_is_visible(ETreeTableAdapter *etta)
