@@ -201,7 +201,7 @@ gchar*	       e_icon_bar_get_item_text		(EIconBar	  *icon_bar,
 						 gint		   item_num);
 void	       e_icon_bar_set_item_text		(EIconBar	  *icon_bar,
 						 gint		   item_num,
-						 gchar		  *text);
+						 const gchar	  *text);
 
 gpointer       e_icon_bar_get_item_data		(EIconBar	  *icon_bar,
 						 gint		   item_num);
