@@ -80,11 +80,19 @@ static void e_shortcut_bar_on_item_added	(EShortcutModel *model,
 						 gint		 item_num,
 						 gchar		*item_url,
 						 gchar		*item_name,
+						 GdkPixbuf      *item_image,
 						 EShortcutBar	*shortcut_bar);
 static void e_shortcut_bar_on_item_removed	(EShortcutModel *model,
 						 gint		 group_num,
 						 gint		 item_num,
 						 EShortcutBar	*shortcut_bar);
+static void e_shortcut_bar_on_item_updated	(EShortcutModel *model,
+						 gint		 group_num,
+						 gint		 item_num,
+						 gchar          *item_url,
+						 gchar          *item_name,
+						 GdkPixbuf      *item_image,
+						 EShortcutBar	*shortcut_bar);
 
 static gint e_shortcut_bar_add_group		(EShortcutBar	*shortcut_bar,
 						 gint		 position,
@@ -95,11 +103,17 @@ static gint e_shortcut_bar_add_item		(EShortcutBar	*shortcut_bar,
 						 gint		 group_num,
 						 gint		 position,
 						 const gchar	*item_url,
-						 const gchar	*item_name);
+						 const gchar	*item_name,
+						 GdkPixbuf      *image);
 static void e_shortcut_bar_remove_item		(EShortcutBar	*shortcut_bar,
 						 gint		 group_num,
 						 gint		 item_num);
-
+static void e_shortcut_bar_update_item		(EShortcutBar 	*shortcut_bar,
+						 gint         	 group_num,
+						 gint         	 item_num,
+						 const gchar  	*item_url,
+						 const gchar  	*item_name,
+						 GdkPixbuf    	*image);
 
 static void e_shortcut_bar_set_canvas_style	(EShortcutBar	*shortcut_bar,
 						 GtkWidget	*canvas);
@@ -133,9 +147,6 @@ static void e_shortcut_bar_on_drag_end		(GtkWidget      *widget,
 						 EShortcutBar   *shortcut_bar);
 static void e_shortcut_bar_stop_editing		(GtkWidget	*button,
 						 EShortcutBar	*shortcut_bar);
-static GdkPixbuf* e_shortcut_bar_get_image_from_url (EShortcutBar *shortcut_bar,
-						     const gchar *item_url);
-static GdkPixbuf* e_shortcut_bar_load_image	(const gchar	*filename);
 
 
 enum
@@ -185,7 +196,7 @@ e_shortcut_bar_class_init (EShortcutBarClass *class)
 	e_shortcut_bar_signals[ITEM_SELECTED] =
 		gtk_signal_new ("item_selected",
 				GTK_RUN_LAST | GTK_RUN_ACTION,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EShortcutBarClass,
 						   item_selected),
 				gtk_marshal_NONE__POINTER_INT_INT,
@@ -195,7 +206,7 @@ e_shortcut_bar_class_init (EShortcutBarClass *class)
 	e_shortcut_bar_signals[SHORTCUT_DROPPED] =
 		gtk_signal_new ("shortcut_dropped",
 				GTK_RUN_LAST | GTK_RUN_ACTION,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EShortcutBarClass, shortcut_dropped),
 				e_shortcut_bar_marshal_NONE__INT_INT_STRING_STRING,
 				GTK_TYPE_NONE, 4,
@@ -207,14 +218,14 @@ e_shortcut_bar_class_init (EShortcutBarClass *class)
 	e_shortcut_bar_signals[SHORTCUT_DRAGGED] =
 		gtk_signal_new ("shortcut_dragged",
 				GTK_RUN_LAST | GTK_RUN_ACTION,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (EShortcutBarClass, shortcut_dragged),
 				gtk_marshal_NONE__INT_INT,
 				GTK_TYPE_NONE, 2,
 				GTK_TYPE_INT,
 				GTK_TYPE_INT);
 
-	gtk_object_class_add_signals (object_class, e_shortcut_bar_signals,
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_shortcut_bar_signals,
 				      LAST_SIGNAL);
 
 	/* Method override */
@@ -276,6 +287,7 @@ e_shortcut_bar_set_model (EShortcutBar *shortcut_bar,
 {
 	gint num_groups, group_num, num_items, item_num;
 	gchar *group_name, *item_url, *item_name;
+	GdkPixbuf *item_image;
 
 	/* Disconnect any existing model. */
 	e_shortcut_bar_disconnect_model (shortcut_bar);
@@ -300,6 +312,9 @@ e_shortcut_bar_set_model (EShortcutBar *shortcut_bar,
 	gtk_signal_connect (GTK_OBJECT (model), "item_removed",
 			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_item_removed),
 			    shortcut_bar);
+	gtk_signal_connect (GTK_OBJECT (model), "item_updated",
+			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_item_updated),
+			    shortcut_bar);
 
 	/* Add any items already in the model. */
 	num_groups = e_shortcut_model_get_num_groups (model);
@@ -313,12 +328,14 @@ e_shortcut_bar_set_model (EShortcutBar *shortcut_bar,
 		for (item_num = 0; item_num < num_items; item_num++) {
 			e_shortcut_model_get_item_info (model, group_num,
 							item_num, &item_url,
-							&item_name);
+							&item_name, &item_image);
 			e_shortcut_bar_add_item (shortcut_bar, group_num,
 						 item_num, item_url,
-						 item_name);
+						 item_name, item_image);
 			g_free (item_url);
 			g_free (item_name);
+			if (item_image != NULL)
+				gdk_pixbuf_unref (item_image);
 		}
 	}
 }
@@ -372,10 +389,11 @@ e_shortcut_bar_on_item_added		(EShortcutModel *model,
 					 gint		 item_num,
 					 gchar		*item_url,
 					 gchar		*item_name,
+					 GdkPixbuf      *item_image,
 					 EShortcutBar	*shortcut_bar)
 {
 	e_shortcut_bar_add_item (shortcut_bar, group_num, item_num,
-				 item_url, item_name);
+				 item_url, item_name, item_image);
 }
 
 
@@ -388,7 +406,40 @@ e_shortcut_bar_on_item_removed (EShortcutModel *model,
 	e_shortcut_bar_remove_item (shortcut_bar, group_num, item_num);
 }
 
+static void
+e_shortcut_bar_on_item_updated (EShortcutModel *model,
+				gint            group_num,
+				gint            item_num,
+				gchar          *item_url,
+				gchar          *item_name,
+				GdkPixbuf      *image,
+				EShortcutBar   *shortcut_bar)
+{
+	e_shortcut_bar_update_item (shortcut_bar, group_num, item_num,
+				    item_url, item_name, image);
+}
 
+static void
+e_shortcut_bar_group_button_press (GtkWidget *widget,
+				   GdkEventButton *button_event,
+				   EShortcutBar *shortcut_bar)
+{
+	EGroupBar *group_bar;
+	int i;
+
+	group_bar = E_GROUP_BAR (shortcut_bar);
+
+	for (i = 0; i < group_bar->children->len; i++) {
+		EGroupBarChild *child;
+
+		child = &g_array_index (group_bar->children, EGroupBarChild, i);
+		if (widget == child->button) {
+			gtk_signal_emit (GTK_OBJECT (shortcut_bar), e_shortcut_bar_signals[ITEM_SELECTED],
+					 button_event, i, -1);
+			break;
+		}
+	}
+}
 
 static gint
 e_shortcut_bar_add_group	(EShortcutBar	*shortcut_bar,
@@ -450,6 +501,10 @@ e_shortcut_bar_add_group	(EShortcutBar	*shortcut_bar,
 #endif
 
 	button = gtk_button_new ();
+	gtk_signal_connect (GTK_OBJECT (button), "button_press_event",
+			    GTK_SIGNAL_FUNC (e_shortcut_bar_group_button_press),
+			    shortcut_bar);
+
 	label = e_entry_new ();
 	gtk_object_set(GTK_OBJECT(label),
 		       "draw_background", FALSE,
@@ -499,10 +554,10 @@ e_shortcut_bar_add_item		(EShortcutBar	*shortcut_bar,
 				 gint		 group_num,
 				 gint		 position,
 				 const gchar	*item_url,
-				 const gchar	*item_name)
+				 const gchar	*item_name,
+				 GdkPixbuf      *image)
 {
 	EShortcutBarGroup *group;
-	GdkPixbuf *image;
 	gint item_num;
 
 	g_return_val_if_fail (E_IS_SHORTCUT_BAR (shortcut_bar), -1);
@@ -511,13 +566,12 @@ e_shortcut_bar_add_item		(EShortcutBar	*shortcut_bar,
 	g_return_val_if_fail (item_url != NULL, -1);
 	g_return_val_if_fail (item_name != NULL, -1);
 
-	image = e_shortcut_bar_get_image_from_url (shortcut_bar, item_url);
 	group = &g_array_index (shortcut_bar->groups,
 				EShortcutBarGroup, group_num);
 
 	item_num = e_icon_bar_add_item (E_ICON_BAR (group->icon_bar),
 					image, item_name, position);
-	gdk_pixbuf_unref (image);
+
 	e_icon_bar_set_item_data_full (E_ICON_BAR (group->icon_bar), item_num,
 				       g_strdup (item_url), g_free);
 
@@ -542,6 +596,37 @@ e_shortcut_bar_remove_item	(EShortcutBar	*shortcut_bar,
 	e_icon_bar_remove_item (E_ICON_BAR (group->icon_bar), item_num);
 }
 
+static void
+e_shortcut_bar_update_item (EShortcutBar *shortcut_bar,
+			    gint          group_num,
+			    gint          item_num,
+			    const gchar  *item_url,
+			    const gchar  *item_name,
+			    GdkPixbuf    *image)
+{
+	EShortcutBarGroup *group;
+	EIconBar *icon_bar;
+	EIconBarItem *item;
+
+	g_return_if_fail (E_IS_SHORTCUT_BAR (shortcut_bar));
+	g_return_if_fail (group_num >= 0);
+	g_return_if_fail (group_num < shortcut_bar->groups->len);
+	g_return_if_fail (item_url != NULL);
+	g_return_if_fail (item_name != NULL);
+
+	group = &g_array_index (shortcut_bar->groups,
+				EShortcutBarGroup, group_num);
+
+	icon_bar = E_ICON_BAR (group->icon_bar);
+
+	item = &g_array_index (icon_bar->items, EIconBarItem, item_num);
+
+	e_icon_bar_set_item_image (icon_bar, item_num, image);
+	e_icon_bar_set_item_text (icon_bar, item_num, item_name);
+
+	e_icon_bar_set_item_data_full (icon_bar, item_num,
+				       g_strdup (item_url), g_free);
+}
 
 static void
 e_shortcut_bar_set_canvas_style (EShortcutBar *shortcut_bar,
@@ -556,9 +641,9 @@ e_shortcut_bar_set_canvas_style (EShortcutBar *shortcut_bar,
         rc_style->fg[GTK_STATE_NORMAL].green = 65535;
         rc_style->fg[GTK_STATE_NORMAL].blue  = 65535;
 
-        rc_style->bg[GTK_STATE_NORMAL].red   = 32512;
-        rc_style->bg[GTK_STATE_NORMAL].green = 32512;
-        rc_style->bg[GTK_STATE_NORMAL].blue  = 32512;
+        rc_style->bg[GTK_STATE_NORMAL].red   = 0x8000;
+        rc_style->bg[GTK_STATE_NORMAL].green = 0x8000;
+        rc_style->bg[GTK_STATE_NORMAL].blue  = 0x8000;
 
         gtk_widget_modify_style (GTK_WIDGET (canvas), rc_style);
         gtk_rc_style_unref (rc_style);
@@ -650,7 +735,8 @@ e_shortcut_bar_on_drag_data_get (GtkWidget          *widget,
 {
 	gchar *data;
 
-	if (info == TARGET_SHORTCUT) {
+	if (info == TARGET_SHORTCUT && shortcut_bar->dragged_name
+	    && shortcut_bar->dragged_url) {
 		data = g_strdup_printf ("%s%c%s", shortcut_bar->dragged_name,
 					'\0', shortcut_bar->dragged_url);
 		gtk_selection_data_set (selection_data,	selection_data->target,
@@ -681,7 +767,7 @@ e_shortcut_bar_on_drag_data_received  (GtkWidget          *widget,
 	position = icon_bar->dragging_before_item_num;
 
 	if ((data->length >= 0) && (data->format == 8)
-	    &&  position != -1) {
+	    && (position != -1) && (info == TARGET_SHORTCUT)) {
 		item_name = data->data;
 		item_url = item_name + strlen (item_name) + 1;
 
@@ -768,61 +854,6 @@ e_shortcut_bar_stop_editing (GtkWidget *button,
 }
 
 
-/* Sets the callback which is called to return the icon to use for a particular
-   URL. */
-void
-e_shortcut_bar_set_icon_callback (EShortcutBar *shortcut_bar,
-				  EShortcutBarIconCallback cb,
-				  gpointer data)
-{
-	shortcut_bar->icon_callback = cb;
-	shortcut_bar->icon_callback_data = data;
-}
-
-
-static GdkPixbuf *
-e_shortcut_bar_get_image_from_url (EShortcutBar *shortcut_bar,
-				   const gchar *item_url)
-{
-	GdkPixbuf *icon = NULL;
-
-	if (shortcut_bar->icon_callback)
-		icon = (*shortcut_bar->icon_callback) (shortcut_bar,
-						       item_url,
-						       shortcut_bar->icon_callback_data);
-
-	if (!icon) {
-		if (!e_shortcut_bar_default_icon_loaded) {
-			e_shortcut_bar_default_icon_loaded = TRUE;
-			e_shortcut_bar_default_icon = e_shortcut_bar_load_image (e_shortcut_bar_default_icon_filename);
-		}
-		icon = e_shortcut_bar_default_icon;
-		/* ref the default icon each time we return it */
-		gdk_pixbuf_ref (icon);
-	}
-
-	return icon;
-}
-
-
-static GdkPixbuf *
-e_shortcut_bar_load_image (const gchar *filename)
-{
-	gchar *pathname;
-	GdkPixbuf *image = NULL;
-
-	pathname = gnome_pixmap_file (filename);
-	if (pathname)
-		image = gdk_pixbuf_new_from_file (pathname);
-	else
-		g_warning ("Couldn't find pixmap: %s", filename);
-
-	g_free (pathname);
-
-	return image;
-}
-
-
 /* Set whether items can be dragged, for drag-and-drop. */
 void
 e_shortcut_bar_set_enable_drags    (EShortcutBar	 *shortcut_bar,
