@@ -21,6 +21,7 @@
 #include <gtk/gtktearoffmenuitem.h>
 #include <gdk/gdkkeysyms.h>
 #include "gtk-combo-box.h"
+#include "gal/util/e-util.h"
 
 static GtkHBoxClass *gtk_combo_box_parent_class;
 static int gtk_combo_toggle_pressed (GtkToggleButton *tbutton,
@@ -38,6 +39,8 @@ static void gtk_combo_tearoff_bg_copy (GtkComboBox *combo);
 enum {
 	POP_DOWN_WIDGET,
 	POP_DOWN_DONE,
+	PRE_POP_DOWN,
+	POST_POP_HIDE,
 	LAST_SIGNAL
 };
 
@@ -109,7 +112,7 @@ gtk_combo_box_class_init (GtkObjectClass *object_class)
 	gtk_combo_box_signals [POP_DOWN_WIDGET] = gtk_signal_new (
 		"pop_down_widget",
 		GTK_RUN_LAST,
-		object_class->type,
+		E_OBJECT_CLASS_TYPE (object_class),
 		GTK_SIGNAL_OFFSET (GtkComboBoxClass, pop_down_widget),
 		my_marshal_POINTER__NONE,
 		GTK_TYPE_POINTER, 0, GTK_TYPE_NONE);
@@ -117,12 +120,28 @@ gtk_combo_box_class_init (GtkObjectClass *object_class)
 	gtk_combo_box_signals [POP_DOWN_DONE] = gtk_signal_new (
 		"pop_down_done",
 		GTK_RUN_LAST,
-		object_class->type,
+		E_OBJECT_CLASS_TYPE (object_class),
 		GTK_SIGNAL_OFFSET (GtkComboBoxClass, pop_down_done),
 		gtk_marshal_BOOL__POINTER,
 		GTK_TYPE_BOOL, 1, GTK_TYPE_OBJECT);
 
-	gtk_object_class_add_signals (object_class, gtk_combo_box_signals, LAST_SIGNAL);
+	gtk_combo_box_signals [PRE_POP_DOWN] = gtk_signal_new (
+		"pre_pop_down",
+		GTK_RUN_LAST,
+		E_OBJECT_CLASS_TYPE (object_class),
+		GTK_SIGNAL_OFFSET (GtkComboBoxClass, pre_pop_down),
+		gtk_marshal_NONE__NONE,
+		GTK_TYPE_NONE, 0);
+
+	gtk_combo_box_signals [POST_POP_HIDE] = gtk_signal_new (
+		"post_pop_hide",
+		GTK_RUN_LAST,
+		E_OBJECT_CLASS_TYPE (object_class),
+		GTK_SIGNAL_OFFSET (GtkComboBoxClass, post_pop_hide),
+		gtk_marshal_NONE__NONE,
+		GTK_TYPE_NONE, 0);
+	
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, gtk_combo_box_signals, LAST_SIGNAL);
 }
 
 static void
@@ -180,6 +199,8 @@ gtk_combo_box_popup_hide_unconditional (GtkComboBox *combo_box)
 	}
 	gtk_object_unref (GTK_OBJECT (combo_box->priv->pop_down_widget));
 	deactivate_arrow (combo_box);
+
+	gtk_signal_emit (GTK_OBJECT (combo_box), gtk_combo_box_signals [POST_POP_HIDE]);
 }
 
 /**
@@ -206,7 +227,7 @@ gtk_combo_box_popup_hide (GtkComboBox *combo_box)
 /*
  * Find best location for displaying
  */
-static void
+void
 gtk_combo_box_get_pos (GtkComboBox *combo_box, int *x, int *y)
 {
 	GtkWidget *wcombo = GTK_WIDGET (combo_box);
@@ -248,6 +269,8 @@ gtk_combo_box_popup_display (GtkComboBox *combo_box)
 		gtk_container_add (GTK_CONTAINER (combo_box->priv->frame), pw);
 	}
 
+	gtk_signal_emit (GTK_OBJECT (combo_box), gtk_combo_box_signals [PRE_POP_DOWN]);
+	
 	if (combo_box->priv->torn_off) {
 		/* To give the illusion that tearoff still displays the
 		 * popup, we copy the image in the popup window to the
@@ -777,3 +800,24 @@ gtk_combo_box_set_arrow_sensitive (GtkComboBox *combo,
 
 	gtk_widget_set_sensitive (combo->priv->arrow_button, sensitive);
 }
+
+/**
+ * gtk_combo_box_set_tearable:
+ * @combo: Combo box
+ * @tearable: whether to allow the @combo to be tearable
+ *
+ * controls whether the combo box's pop up widget can be torn off.
+ */
+void
+gtk_combo_box_set_tearable (GtkComboBox *combo, gboolean tearable)
+{
+	g_return_if_fail (combo != NULL);
+	g_return_if_fail (GTK_IS_COMBO_BOX (combo));
+
+	if (tearable){
+		gtk_widget_show (combo->priv->tearable);
+	} else {
+		gtk_combo_set_tearoff_state (combo, FALSE);
+		gtk_widget_hide (combo->priv->tearable);
+	}
+}
